# Copyright 2019-2023 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from charms.reactive import when_any, Endpoint
from ols import postgres


relations = postgres.db_relations()
connected = tuple('{}.connected'.format(r) for r in relations)


# all the requires side needs to to do is set_database and set_roles
# this is idempotent, so we just always do it
@when_any(*connected)
def set_db_config():
    for state in connected:
        pgsql = Endpoint.from_flag(state)
        if pgsql is None:
            continue

        cfg = relations[pgsql.endpoint_name]
        name = cfg.get('name')
        if name:
            pgsql.set_database(cfg['name'])
            roles = cfg.get('roles')
            if roles:
                pgsql.set_roles(roles)
