# Copyright 2022 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import subprocess

from charms.launchpad.payload import ensure_home_dir
from charms.layer import options
from charms.reactive import clear_flag, hook, set_flag, when, when_not
from ols import base


# Monkey-patch layer:ols.
def create_virtualenv(wheels_dir, codedir, python_exe):
    build_target = (
        options().get("launchpad-payload", {}).get("build_target", "compile")
    )
    subprocess.run(
        ["make", build_target, f"PYTHON={python_exe}"],
        cwd=codedir,
        check=True,
    )


base.create_virtualenv = create_virtualenv


@when("ols.configured")
@when_not("launchpad.payload.configured")
def configure():
    ensure_home_dir()
    set_flag("launchpad.payload.configured")


@when("launchpad.payload.configured")
@when_not("ols.configured")
def deconfigure():
    clear_flag("launchpad.payload.configured")


@hook("upgrade-charm")
def upgrade_charm():
    clear_flag("launchpad.payload.configured")


@when("config.changed.build_label")
def build_label_changed():
    clear_flag("ols.service.installed")
    clear_flag("ols.configured")
